/* ***** BEGIN LICENSE BLOCK *****

 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * The Original Code is bookmarkshome
 *
 * The Initial Developer of the Original Code is Jeroen Groenenboom.
 *
 * Contributor(s):
 *   Dimiter Dimitrov <dimiter@blue-edge.bg>
 *   Josh Powell <Josh@ki4bbo.org> 
 *   Jean-Bernard Marcon <jean-bernard.marcon@laposte.net>
 *
 * ***** END LICENSE BLOCK ***** */

// constructor for column
function bmhColumnTd() {
    this.node = document.createElement( "td" );
    this.offsetNode = document.createElement( "div" );
    this.node.appendChild( this.offsetNode );
}

bmhColumnTd.prototype = {

isEmpty: function() {
	     return this.offsetNode == this.node.firstChild;
	 },


getOffset: function() {
	       return this.offsetNode.offsetTop;
	   },

add: function( folderDiv ) {
	 this.node.insertBefore( folderDiv.node, this.offsetNode );
     }
}

// constructor for folder
function bmhFolderDiv( ancestors, index ) {
    this.node = document.createElement( "div" );
    this.folderUl = document.createElement( "ul" );
    this.node.setAttribute( "class", "folder" );
    // folder h1
    var folderH1 = document.createElement( "h1" );
    this.node.appendChild( folderH1 );
    // folder anchor
    var folderA = document.createElement( "a" );
    folderA.setAttribute( "class", "headerLink" );
    var listId = "folderList" + index;
    folderA.setAttribute( "href", "javascript:void(0)" );
    folderA.setAttribute( "onclick", "void(bmhCheck('openFolder','" + listId + "'))" );
    this.createHeaderText( ancestors, folderA ); 
    folderH1.appendChild( folderA );
    // folder ul
    this.folderUl.setAttribute( "id", listId );
    this.node.appendChild( this.folderUl );
}


bmhFolderDiv.prototype = {

isEmpty: function() {
	     return( this.folderUl.getElementsByTagName( "a" ).length == 0 );
	 },

	 // create folder header based on maximum header lines pref
createHeaderText: function( ancestors, folderA ) {
		      var headerArray = eval( ancestors.toSource() );
		      var top = headerArray.length - bmhLib.getPref( "maxHeaderLines" );
		      var i = top;
		      if( top > 1 ) {
			  // at least one ancestor won't be displayed
			  headerArray[top] = "> ".concat( headerArray[top] );
			  i--;
			  // append '>' for other ancestors that won't be displayed
			  for( ; i > 1; i-- ) {
			      headerArray[top] = ">".concat( headerArray[top] );
			  }
		      }
		      // top <= 1 -> all ancestors (except root) will be displayed
		      i = ( top <= 1 ? 1 : top );
		      // root ancestor only displayed for root folder
		      if( ancestors.length == 1 ) {
			  i = 0;
		      }
		      var folderName, headerBr, span;
		      for( ; i < headerArray.length; i++ ) {
			  folderName = document.createTextNode( this.processName( headerArray[i] ));	
			  span = document.createElement( "span" );
			  folderA.appendChild( span );
			  span.appendChild( folderName );
		      }
		  },

add: function( name, url, description, icon, type ) {
	 // folder li
	 var folderLi = document.createElement( "li" );
	 this.folderUl.appendChild( folderLi );
	 // folder a
	 var folderA = document.createElement( "a" );
	 folderA.setAttribute( "href", url );
	 var title = this.getTitle( name, description );
	 if( title ) {
	     folderA.setAttribute( "title", title );
	 }
	 folderLi.appendChild( folderA );
	 var aName = this.processName( name );
	 // text a
	 var aText = document.createTextNode( aName );
	 // icons, list style icons doesn't work
	 var image;
	 if( icon != "" ) {
	     image = document.createElement( "img" );
	     image.setAttribute( "src", icon );
	     image.setAttribute( "alt", "" );
	     if( type == "Folder" ) {
		 image.setAttribute( "class", "normalIcon" );
	     }
	     else if( type == "Livemark" ) {
		 image.setAttribute( "class", "LivemarkIcon" );
	     }
	     // display depends on prefs
	     image.style.display = bmhLib.getIconDisplay( type, bmhLib.getPref( "iconDisplay" ));
	     folderA.appendChild( image );
	 }
	 folderA.appendChild( aText ); 
     },

addSeparator: function() {
		  var folderLi = document.createElement( "li" );
		  folderLi.setAttribute( "class", "separator" );
		  this.folderUl.appendChild( folderLi );
	      },

getTitle: function( name, description ) {
	      return name.length > bmhLib.getPref( "maxNameLength" )
		  ? name
		  : null;
	  },

processName: function( name ) {
		 var displayName = name;
		 var reg1 = /^(?:\w+:[\/\\]{1,3}(?:www\.)?)/; 
		 var reg2 = /.{2}$/;	
		 var max = bmhLib.getPref( "maxNameLength" );
		 if( displayName.length > max ) {
		     // remove protocol of url
		     displayName = displayName.replace( reg1, "" );
		     if( displayName.length > max ) {
			 displayName = displayName.substring( 0, max );
			 // show that name is longer than portrayed
			 displayName = displayName.replace( reg2, ".." );
		     }
		 }
		 return displayName;
	     }
}


function bmhFolder( node, ancestors, type ) {
    this.node = node;
    this.type = type;
    // ancestors include name of folder
    this.ancestors = ancestors;
}

var bmh = {

main: function() {
	  // localizations in document
	  bmh.localize();
	  // RDF  
	  var rdf = new BmhRDF();
	  var curName = bmhLib.getProperty( rdf.bmRoot, rdf.NameArc, rdf.BMDS );
	  var curFolder = new bmhFolder( rdf.bmRoot, [curName], "Folder" );
	  var folderStack = new Array( curFolder );
	  var inOrExcludedFolders = bmhLib.getArrayFromPref( "inOrExcludedFolders" );
	  var enumerator, tempFolders, curNode;
	  var curType, curURL, curDesc, curIcon, curNodeAncestors;
	  // search engines
	  bmh.createEngineMenu( rdf );
	  bmh.showHideSearchBar();
	  // add style 
	  bmh.addStyle( rdf );
	  
	  var mainRow = document.getElementsByTagName( "tr" )[0];
	  var columns = bmhLib.getPref( "nrOfColumns" ); 
	  var columnTds = new Array( columns );
	  // add preferred nr of columns to page
	  var i;
	  for( i = 0; i < columnTds.length; i++ ) {
	      columnTds[i] = new bmhColumnTd();
	      mainRow.appendChild( columnTds[i].node );
	  }
	  var folderDiv;
	  var isExcluded, folderIndex = 0;
	  // make page
	  while( folderStack.length > 0 ) {
	      curFolder = folderStack.pop();
	      folderIndex++;
	      folderDiv = new bmhFolderDiv( curFolder.ancestors, folderIndex );
	      rdf.RDFC.Init( rdf.BMDS, curFolder.node );
	      enumerator = rdf.RDFC.GetElements();
	      tempFolders = new Array();
	      while( enumerator.hasMoreElements() ) {
		  curNode = enumerator.getNext();
		  curType = bmhLib.getProperty( curNode, rdf.typeArc, rdf.BMDS )
		      .split( "#" )[1]; 
		  curName = bmhLib.getProperty( curNode, rdf.NameArc, rdf.BMDS );
		  if( curType == "Bookmark" ) {
		      // Bookmark
		      curURL = bmhLib.getProperty( curNode, rdf.URLArc, rdf.BMDS ); 
		      curDesc = bmhLib.getProperty( curNode, rdf.DescriptionArc, rdf.BMDS );
		      curIcon = bmhLib.getProperty( curNode, rdf.IconArc, rdf.BMDS );
		      // icon in link of bmh page is not picked up by FF 
		      if( /(^chrome)|(^about).*bookmarkshome/.test( curURL )) {
			  curIcon = document.getElementsByTagName( "link" )[0].getAttribute( "href" );
		      }
		      folderDiv.add( curName, curURL, curDesc, curIcon, curFolder.type );
		  }
		  else if( curType == "Folder"  || curType == "Livemark" ) {
		      // Folder
		      curNodeAncestors = curFolder.ancestors.concat( [curName] );
		      tempFolders.push( new bmhFolder( curNode, curNodeAncestors, curType ));
		  }
		  else if( curType == "BookmarkSeparator" ) {
		      folderDiv.addSeparator();
		  }
	      }
	      isExcluded = false;
	      isExcluded = bmhLib.isExcludedFolder( curFolder.ancestors.toString(),
		      bmhLib.getPref( "excludeIndex" ), inOrExcludedFolders );
	      // check if folder contains bookmarks and is not excluded
	      // Display empty livemarks
	      if(( ! folderDiv.isEmpty() || curFolder.type == "Livemark" ) && ! isExcluded ) {
		  i = bmh.smallestColumn( columnTds );
		  // add folder to smallest column
		  columnTds[i].add( folderDiv );
	      }
	      // reverse to match order in bookmarks tree
	      folderStack = folderStack.concat( tempFolders.reverse() );
	  }
	  // remove empty columns
	  for( i = 0; i < columnTds.length; i++ ) {
	      if( columnTds[i].isEmpty() ) {
		  mainRow.removeChild( columnTds[i].node );
	      }
	  }
      },

localize: function() {
	      // insert element with text for alert statement in script of document
	      var alertSpan = document.createElement( "span" );
	      alertSpan.setAttribute( "id", "alertSpan" );
	      alertSpan.style.display = "none";
	      document.getElementById( "otherLinks" ).appendChild( alertSpan );
	      // submit button and links at bottom of page
	      var readmeLink = document.getElementById( "readmeLink" );
	      var homeLink = document.getElementById( "homeLink" );
	      var optionsLink = document.getElementById( "optionsLink" );
	      var submitInput = document.getElementById( "submitInput" );
	      // alert text
	      var alertText = document.createTextNode( bmhLib.getLocaleString( "NotAvailableString" ));
	      alertSpan.appendChild( alertText );
	      document.getElementById( "otherLinks" ).appendChild( alertSpan );
	      // submit button text
	      var submitText = document.createTextNode( bmhLib.getLocaleString( "SearchButtonString" ));
	      submitInput.value = submitText.nodeValue;
	      // readme text 
	      var readmeText = document.createTextNode( bmhLib.getLocaleString( "ReadmeString" ));
	      readmeLink.appendChild( readmeText );
	      // home text 
	      var homeText = document.createTextNode( bmhLib.getLocaleString( "HomePageString" ));
	      homeLink.appendChild( homeText );
	      // options text
	      var optionsText = document.createTextNode( bmhLib.getLocaleString( "OptionsString" ));
	      optionsLink.appendChild( optionsText );
	  },


showHideSearchBar: function() {
		       var searchForm = document.getElementById( "searchForm" );
		       if( bmhLib.getPref( "hideSearchEngines" )) {
			   searchForm.style.display = "none";
		       }
		       if( searchForm.style.display != "none" ) {
			   document.getElementById( "queryInput" ).focus();
		       }
		   },

smallestColumn: function( columns ) {
		    var index = 0;
		    for(var i = 1; i < columns.length; i++ ) {
			if( columns[i].getOffset() < columns[index].getOffset() ) {
			    index = i;
			}
		    }
		    return index;
		},

showDefaultEngine: function( index ) {
		       document.getElementById( "engineSelect" ).selectedIndex = index;
		   },

addStyle: function( rdf ) {
	      try {
		  // get styles datasource
		  var stylesDS = bmhLib.getStylesDS( rdf );
		  // get default style
		  var defaultStyle = rdf.RDF.GetResource( rdf.stylesNS + bmhLib.getPref( "styleName" ));
		  //var sheetArc = rdf.RDF.GetResource( f.stylesNS + "sheet" );
		  var defaultSheet = bmhLib.getProperty( defaultStyle, rdf.sheetArc, stylesDS );
		  // insert stylesheet in document 
		  var styleTag = document.getElementsByTagName( "style" )[0];
		  styleTag.innerHTML = bmhLib.formatSheet( defaultSheet );
	      }
	      catch( e ) {}
	  },

openSidebar: function() {
		 try {
		     var mainDoc = bmhLib.getMainWindow().document;
		     var bmhBroadcasterId = "viewBmhPrefs";
		     var bmhBroadcaster = mainDoc.getElementById( bmhBroadcasterId );
		     var sidebar = mainDoc.getElementById( "sidebar" );
		     var sidebarBox = mainDoc.getElementById( "sidebar-box" );
		     var sidebarTitle = mainDoc.getElementById( "sidebar-title" );
		     var sidebarSplitter = mainDoc.getElementById( "sidebar-splitter" );
		     var sidebarBroadcasters = mainDoc.getElementsByAttribute( "group", "sidebar" );
		     for (var i = 0; i < sidebarBroadcasters.length; i++ ) {
			 sidebarBroadcasters[i].removeAttribute( "checked" );
		     }
		     bmhBroadcaster.setAttribute( "checked", "true" );
		     if( sidebarBox.hidden ) {
			 sidebarBox.hidden = false;
			 sidebarSplitter.hidden = false;
		     }
		     var url = bmhBroadcaster.getAttribute( "sidebarurl" );
		     var title = bmhBroadcaster.getAttribute( "sidebartitle" );
		     if (!title) {
			 title = bmhBroadcaster.getAttribute( "label" );
		     }
		     sidebar.setAttribute( "src", url );
		     sidebarBox.setAttribute( "src", url );
		     sidebarBox.setAttribute( "sidebarcommand", bmhBroadcaster.id );
		     sidebarTitle.setAttribute( "value", title );
		 }
		 catch( e ) {}
	     },

createEngineMenu: function( rdf ) {
		      try {
			  var engineTargets = rdf.ISDS.GetTargets( rdf.rootEngine, rdf.childArc, true );
			  var curNode, curName, curIcon, searchURL, curOption, hiddenIndex, curTextNode;
			  var engineSelect = document.getElementById( "engineSelect" );
			  var dfltEngineValue = bmhLib.getPref( "engineURL" );
			  var hiddenEngines = bmhLib.getArrayFromPref( "hiddenEngines" );
			  var dfltOption = null;
			  var newHiddenEngines = [];
			  while ( engineTargets.hasMoreElements()) {
			      curNode = engineTargets.getNext().QueryInterface( Components.interfaces.nsIRDFResource );
			      curName = bmhLib.getProperty( curNode, rdf.NameArc, rdf.ISDS );
			      curIcon = bmhLib.getProperty( curNode, rdf.IconArc, rdf.ISDS );
			      try {
				  searchURL = rdf.ISEARCHSVC.GetInternetSearchURL( curNode.Value, "bmh", 0, 0, {value:0} );
			      }
			      catch( e ) {
				  searchURL = "invalid searchurl";
			      }
			      curOption = document.createElement( "option" );
			      curOption.value = curNode.Value + "|" + curIcon + "|" + searchURL;
			      // determine default engine
			      if( curNode.Value == dfltEngineValue ) {
				  dfltOption = curOption;
			      }
			      hiddenIndex = bmhLib.binarySearch( curNode.Value, hiddenEngines );
			      // check if engine is hidden
			      if( hiddenIndex >= 0 ) {
				  curOption.style.display = "none";
				  // newHidden is hidden without removed engines
				  newHiddenEngines = newHiddenEngines.concat( hiddenEngines.splice( hiddenIndex, 1 ));
			      }
			      curTextNode = document.createTextNode( curName );
			      curOption.appendChild( curTextNode );
			      engineSelect.appendChild( curOption );
			  }
			  bmhLib.setPref( "hiddenEngines", 32, bmhLib.getPrefFromArray( newHiddenEngines )); 
			  bmhLib.selectEngine( dfltOption );
		      }
		      catch( e ) {}
		  }
}



